/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GRAPHWIDGET_H_
#define _GRAPHWIDGET_H_

#include<gtkmm.h>
#include"graphmodel.h"
#include<string>

class TGraphWidget;
    
/**
 * This class implements the graph painting widget.
 * TGraphArea is working tightly bound to the TGraphModel.
 * The points are connected to the graph lines using 
 * the spline-interpolation (the interpolation precision 
 * is set by the property, the linear interpolation is the
 * default). The TGraphArea provides the possibility 
 * to specify the data model, the axes scale, etc.
 * This widget should not be layed out directly - place
 * TGraphWidget on a form instead.
 */
class TGraphArea: public Gtk::DrawingArea {
  public:    
    TGraphArea();
    virtual ~TGraphArea();
    void set_master(TGraphWidget * ms);
  protected:
    TGraphWidget * master;
    virtual void on_realize();
    virtual bool on_expose_event(GdkEventExpose* e);
    virtual void on_size_request(Gtk::Requisition* requisition);
  private:
    Glib::RefPtr<Gdk::GC> gc;
    Gdk::Color red_;
    Gdk::Color black_;
    Gdk::Color white_;
    Gdk::Color blue_;
};

/**
 * This class is the container for the TGraphArea.
 * It creates the graph area automatically and provides 
 * the widget scrolling.
 */
class TGraphWidget : public Gtk::Table
{
    public:
        TGraphWidget(TGraphModel * md);
        virtual ~TGraphWidget();
        void setModel(TGraphModel * md);
		
        TGraphModel * model;
        void invalidateGraphState();
    
        double grid_x_start_line;
        double grid_y_start_line;
        
        virtual void on_size_allocate(Gtk::Allocation& allocation);

        // public getters and setters
        const bool get_auto_scale_x();
        void set_auto_scale_x(const bool new_value);
        const bool get_auto_scale_y();
        void set_auto_scale_y(const bool new_value);
        const double get_scale_x();
        void set_scale_x(const double new_value);
        const double get_scale_y();
        void set_scale_y(const double new_value);
        const double get_start_x();
        void set_start_x(const double new_value);
        const double get_start_y();
        void set_start_y(const double new_value);
        const double get_grid_x_step();
        void set_grid_x_step(const double new_value);
        const double get_grid_y_step();
        void set_grid_y_step(const double new_value);
        const double get_axe_x_step();
        void set_axe_x_step(const double new_value);
        const double get_axe_y_step();
        void set_axe_y_step(const double new_value);
        const std::string get_axe_x_label_format();
        void set_axe_x_label_format(const std::string new_value);
        const std::string get_axe_y_label_format();
        void set_axe_y_label_format(const std::string new_value);
        const int get_inset_x();
        void set_inset_x(int new_value);
        const int get_inset_y();
        void set_inset_y(int new_value);
    protected:
        Gtk::HScrollbar* hScrollbar;
        Gtk::VScrollbar* vScrollbar;
        TGraphArea* drawArea;
    
        // this flag should be set to true to force the 
        // graph state recalculation through invalidateGraphState
        bool forceInvalidate;

        bool auto_scale_x;
        bool auto_scale_y;
        double scale_x;
        double scale_y;
        double start_x;
        double start_y;
        double grid_x_step;
        double grid_y_step;
        double axe_x_step;
        double axe_y_step;
        std::string axe_x_label_format;
        std::string axe_y_label_format;
        int inset_x;
        int inset_y;
};


#endif	//_GRAPHWIDGET_H_
